{****************************************
**  TmsResourceItem version 1.0        **
**        (c) 2009 MarcaSoft           **
****************************************}

unit msResourceItem;

interface

uses
  SysUtils, Classes;

type

 TmsResourceItem = class(TComponent)
 private
  fFileName: tfilename;
  FData: Pointer;
  FDataSize: Integer;
  procedure WriteData(Stream: TStream);
  procedure ReadData(Stream: TStream);
  procedure LoadFromStream(S: TStream);
  procedure LoadFromFile(const FileName: TFileName);
  function Empty: Boolean;
  function Equal(ResIt: TmsResourceItem): Boolean;
  procedure SetFileName(aFileName: TfileName);
 protected
  procedure DefineProperties(Filer: TFiler); override;
  procedure SetName(const NewName: TComponentName); override;
 public
  constructor Create(AOwner:TComponent); override;
  destructor Destroy; override;
  procedure SaveToFile(const FileName: TFileName);
  procedure SaveToStream(S: TStream);
 published
  property SourceFileName: TFileName
   read fFileName write SetFilename;
 end;



procedure Register;


implementation

uses MMSystem, Windows;

{ TmsResourceItem }

procedure Register;
begin
  RegisterComponents('MarcaSoft', [TmsResourceItem]);
end;

constructor TmsResourceItem.Create(AOwner:TComponent);
begin
  inherited Create(AOwner);
end;


destructor TmsResourceItem.Destroy;
begin
 if not Empty then
  FreeMem(FData, FDataSize);
 inherited Destroy;
end;

procedure TmsResourceItem.SetName(const NewName: TComponentName);
begin
  inherited;
end;

procedure TmsResourceItem.SetFileName(aFileName: tfilename);
begin
  if csdesigning in componentstate then begin
    if fileexists(afilename) then begin
      ffilename:=afilename;
      LoadFromFile(ffilename);
    end else begin
      ffilename:='(None)';
     if not Empty then
      FreeMem(FData, FDataSize);
     FDataSize := 0;
   end;
  end;
end;


function StreamsEqual(S1, S2: TMemoryStream): Boolean;
begin
 Result := (S1.Size = S2.Size) and CompareMem(S1.Memory, S2.Memory, S1.Size);
end;

procedure TmsResourceItem.DefineProperties(Filer: TFiler);
 function DoWrite: Boolean;
 begin
  if Filer.Ancestor <> nil then
   Result := not (Filer.Ancestor is TmsResourceItem) or
    not Equal(TmsResourceItem(Filer.Ancestor))
  else
   Result := not Empty;
 end;

begin
 inherited DefineProperties(Filer);
 Filer.DefineBinaryProperty('Data', ReadData, WriteData, DoWrite);
end;

function TmsResourceItem.Empty: Boolean;
begin
 Result := FDataSize = 0;
end;

function TmsResourceItem.Equal(ResIt: TmsResourceItem): Boolean;
var
 MyImage, ResItImage: TMemoryStream;
begin
 Result := (ResIt <> nil) and (ClassType = ResIt.ClassType);
 if Empty or ResIt.Empty then
 begin
  Result := Empty and ResIt.Empty;
  Exit;
 end;
 if Result then
 begin
  MyImage := TMemoryStream.Create;
  try
   SaveToStream(MyImage);
   ResItImage := TMemoryStream.Create;
   try
    ResIt.SaveToStream(ResItImage);
    Result := StreamsEqual(MyImage, ResItImage);
   finally
    ResItImage.Free;
   end;
  finally
   MyImage.Free;
  end;
 end;
end;

procedure TmsResourceItem.LoadFromFile(const FileName: TFileName);
var
 F: TFileStream;
begin
 F := TFileStream.Create(FileName, fmOpenRead);
 try
  LoadFromStream(F);
 finally
  F.Free;
 end;
end;

procedure TmsResourceItem.LoadFromStream(S: TStream);
begin
 if not Empty then
  FreeMem(FData, FDataSize);
 FDataSize := 0;
 FData := AllocMem(S.Size);
 FDataSize := S.Size;
 S.Read(FData^, FDataSize);
end;

procedure TmsResourceItem.ReadData(Stream: TStream);
begin
 LoadFromStream(Stream);
end;

procedure TmsResourceItem.SaveToFile(const FileName: TfileName);
var
 F: TFileStream;
begin
 if not Empty then begin
   F := TFileStream.Create(FileName, fmCreate);
   try
    SaveToStream(F);
   finally
    F.Free;
   end;
  end;
end;

procedure TmsResourceItem.SaveToStream(S: TStream);
begin
 if not Empty then
  S.Write(FData^, FDataSize);
end;

procedure TmsResourceItem.WriteData(Stream: TStream);
begin
 SaveToStream(Stream);
end;

end.

